% Tuning, simulation and testing of MISO PI controller on MIMO system.
% This script will tunes and implement a MISO Temperature controller with 
% Fan and Bulb as MV's.
%
% The MISO control structure consist of the two SISO controllers.
% The controllers in the MISO control can be tuned more 
% agressive than the SISO controllers because the two inputs has gains with
% different signs and any overshoot will be coorected by the other 
% controller. Setpoint back-off and alternative smooth controllers
% near setpoint is also implemented.
% The script simulate the controllers in simMISOcontrol.slx and test it in
% realtime in runMISOcontrol.slx
%
% See also SIMCPI SISOCONTROL

% Author: Sindre Johan Heggheim, sindrjh@stud.ntnu.no, November 21th, 2017
% Specialization project TKP4580 at Department of Chemical Engineering

%% Initialization 

clear
close all
clc

% The script uses runSystem.slx to run the MIMO system that is connected
% with the port spesified as teh 'com' variable. 
% tom1ar8_lib.mdl, tom_lib.mdl and msfun_realtime_pacer.m is also needed
% to run the system.
Ts = 0.1;
com = 'COM4';   % com port
delete(instrfind({'Port'},{com}));
baud = 250000;  % baud rate 
Ts  = 0.1;      % Sampling time period
fTt = 0.05;     % Filter time constant for temperature (0.05s - 10s)
fTl = 0.05;     % Filter time constant for light intensity (0.05s - 10s)
fTf = 0.1;      % Filter time constant for light intensity (0.1s - 10s)
t_sim = 800;    % Simulation time

% Simulink variables:
y0 = 0;             % Shifting temperature

% Disturbances
stepTime_d1 = 0;
step_d1= 0;
stepTime_d2 = 0;
step_d2 = 0;

%setpoints:
y1_s0 = 0;
stepTime_ys1 = 0;
step_ys1 = 0;

y2_s0 = 0;
stepTime_ys2 = 0;
step_ys2 = 0;

% tresholds alternative controllers:
altu1_threshold = 0;
altu2_threshold = 0;

%% Load response models and tune controllers:
load('WS_tfs')  

% System transfer functions:
sys_tf11 = tf_BT;
sys_tf21 = tf_FT;
sys_num11 = cell2mat(sys_tf11.num);        
sys_den11 = cell2mat(sys_tf11.den);
sys_num21 = cell2mat(sys_tf21.num);        
sys_den21 = cell2mat(sys_tf21.den);

% calculate tight PI control parameters
[Kc11,tauI11] = SIMCPI(sys_tf11,0.1);   % tauc = 0.1
[Kc21,tauI21] = SIMCPI(sys_tf21,1);     % tauc = 1

%% Test controller on estimated system
y0 = 0;
y_s0 = 0;
stepTime_ys1 = 30; 
stepTime_ys2 = 60;
step_ys1 = 5;
step_ys2 = -5;  
stepTime_d1 = 90;
stepTime_d2 = 120;
step_d1 = -2.5;
step_d2 = +2.5;
t_sim = 150;

% Run simulation:
open('simMISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotMISOcontrol(y,u,ys,IAE,time);
% figIm = getframe(gcf);
% imwrite (figIm.cdata, 'MISOsimT.png', 'png');
% hgexport(fig,'MISOsimT.eps')

close_system('simMISOcontrol');

%% Test controller on real system
% measure and shift with standby temperature

t_step = 0; % no step 
t_sim  = 2;
stepBulb = 0;
stepFan = 0;
stepLed = 0;
open('runSystem');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS
sb_T = Temperature(1,1); %standby temperature

sb_T = 27;
save_system('runSystem')
close_system('runSystem');

% Shift temperature:
y0 = sb_T+3;
t_sim = 150;
bo_c21 = +0.15; % Back-off on Fan
% The real system will always have error and it is not possible that both
% controllers pushes to the same setpoint from both directions. The result
% will be a very high input usage and oscillations. The back off helps the
% system to choose one of the two setpoints and be controlled by just one
% of the controllers.

% Choose IO
turnOn_u1 = 1; % Bulb in
turnOn_u2 = 1; % Fan in
turnOn_u3 = 0;
turnOn_y1 = 1; % Temp out
turnOn_y2 = 0;
fixed_u1 = 0;
fixed_u2 = 0;
fixed_u3 = 0;

open('runMISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotMISOcontrol(y,u,ys,IAE,time);
% figIm = getframe(gcf);
% imwrite (figIm.cdata, 'MISOtestT.png', 'png');
% hgexport(fig,'MISOtestT.eps')

%% Test controller on real system:
% Lowering the oscillations by decreasing the back-off in setpoint and 
% adding alternative controllers with smooth control near the setpoint

bo_c21 = +0.1; %  Back-off on Fan
altu1_threshold = 0.2;
altu2_threshold = 0.2;

[Kc11,tauI11] = SIMCPI(sys_tf11,0.1);
[Kc21,tauI21] = SIMCPI(sys_tf21,1);
% alternative controllers has 10 times larger tauc and the backoff on the 
[alt_Kc11,alt_tauI11] = SIMCPI(sys_tf11,1);
[alt_Kc21,alt_tauI21] = SIMCPI(sys_tf21,10); 

open('alt_runMISOControl');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotMISOcontrol(y,u,ys,IAE,time);
figIm = getframe(gcf);
imwrite (figIm.cdata, 'MISOtestT_alt.png', 'png');
hgexport(fig,'MISOtestT_alt.eps')   
