% Tuning, simulation and testing of SISO PI controllers in MIMO system
% Temperature can be controlled by the Bulb and Fan and the Light Intensity 
% can be controlled by the Bulb and LED. All delays (theta) are neglected 
% and the derivitive part is not needed.
% 
% The PI controllers PI: c(s) = K_c*( 1 + 1/(tau_I*s)) are tuned with 
% SIMC PI tuning rules. The transfer functions used in the tuning was 
% calculated in identification.m and saved as WS_tfs.mat
% The controllers in simulink uses Clamping as Anti-windup method.
% The script simulate the controllers in simSISOcontrol.slx and test it in
% realtime in runSISOcontrol.slx
%
% See also SIMCPI SIMCPI_nosat IDENTIFICATION

% Author: Sindre Johan Heggheim, sindrjh@stud.ntnu.no, November 20th, 2017
% Specialization project TKP4580 at Department of Chemical Engineering

%% Initialization 

clear
close all
clc

% The script uses runSystem.slx to run the MIMO system that is connected
% with the port spesified as teh 'com' variable. 
% tom1ar8_lib.mdl, tom_lib.mdl and msfun_realtime_pacer.m is also needed
% to run the system.
com = 'COM4';   % com port
delete(instrfind({'Port'},{com}));
baud = 250000;  % baud rate 
Ts = 0.1;       % Sampling time period
fTt = 0.05;     % Filter time constant for temperature (0.05s - 10s)
fTl = 0.05;     % Filter time constant for light intensity (0.05s - 10s)
fTf = 0.1;      % Filter time constant for light intensity (0.1s - 10s)
t_sim = 400;    % Simulation time

load('WS_tfs')  % estimated transfer functions 

%% Using Bulb as Temperature controller. 
% Temperature is controlled by a SISO controller on the Bulb and the
% Light Intensity is not taken into any consideration.

% System transfer function:
sys_tf = tf_BT;
sys_num = cell2mat(sys_tf.num);        
sys_den = cell2mat(sys_tf.den);         

% Calculate smooth SIMC PI control parameters with so saturation on a 5
% degree step.
ys_max = 5;                         % max step in y without saturation.
[Kc,tauI,tauc] = SIMCPI_nosat(sys_tf,ys_max);

% Test controller on estimated system
y0 = 0;             % Shifting of y
y_s0 = y0;          % initial set-point
t_sim = 800;        % Simulation time
stepTime1 = 200;    % Time when first step
stepTime2 = 0;      % No step
step_ys1 = 5;       % Step in setpoint 
step_ys2 = -0;      % No step
stepTime_d1 = 400;  % Time disturbance 1
stepTime_d2 = 600;  % Time disturbance 2
step_d1 = -2.5;     % Value of disturbance 1
step_d2 = +2.5;     % Value of disturbance 2

% Run simulation:
open('simSISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotSISOcontrol(y,u,ys,IAE,time);
figIm = getframe(gcf);
imwrite (figIm.cdata, 'SISOsimBT.png', 'png');
hgexport(fig,'SISOsimBT.eps')

%% Test Bulb-Temperature controller on real system:
% measure and shift with standby temperature
    t_step = 0; % no step
    t_sim  = 2;
    stepBulb = 0;
    stepFan = 0;
    stepLed = 0;
    open('runSystem');
    set_param(gcs,'StopTime',num2str(t_sim));
    set_param(gcs,'SimulationCommand','start');
    while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
        pause(1)
    end%while
    pause(1) %Waiting for simulink to save to WS
sb_T = Temperature(1,1); %standby temperature
sb_T = 27;   % most measured standby temperature
close_system('runSystem');

% Response and rejection:
y0 = sb_T;          % shift temperature
t_sim = 800;        % Simulation time

% Choose IO and fixed inputs in simulink system model
turnOn_u1 = 1; % Bulb in
turnOn_u2 = 0;
turnOn_u3 = 0;
turnOn_y1 = 1; % Temp out
turnOn_y2 = 0;
fixed_u1  = 0;
fixed_u2  = 0;
fixed_u3  = 0;

% Test controller:
open('runSISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotSISOcontrol(y,u,ys,IAE,time);
% figIm = getframe(gcf);
% imwrite (figIm.cdata, 'SISOtestBT.png', 'png');
% hgexport(fig,'SISOtestBT.eps')

%% Using Fan as temperature controller. 
% Temperature is controlled by a SISO controller on the Fan and the
% Light Intensity is not taken into any consideration.

% System transfer function:
sys_tf = tf_FT;
sys_num = cell2mat(sys_tf.num);        
sys_den = cell2mat(sys_tf.den);         

% Calculate smooth SIMC PI control parameters with so saturation on a 5
% degree step.
% [Kc,tauI] = SIMCPI(sys_tf,0);
[Kc,tauI,tauc] = SIMCPI_nosat(sys_tf,5); 

% Test controller on estimated system
y0 = 0;             % Shifting of y
y_s0 = 0;           % initial set-point
t_sim = 800;        % Simulation time
stepTime1 = 0;      % no step
stepTime2 = 200;    % Time when first step
step_ys1 = 0;       % No step
step_ys2 = -5;      % Step in setpoint
stepTime_d1 = 400;  % Time disturbance 1
stepTime_d2 = 600;  % Time disturbance 2
step_d1 = -2.5;     % Value of disturbance 1
step_d2 = +2.5;     % Value of disturbance 2

% Run simulation:
open('simSISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotSISOcontrol(y,u,ys,IAE,time);
figIm = getframe(gcf);
imwrite (figIm.cdata, 'SISOsimFT.png', 'png');
hgexport(fig,'SISOsimFT.eps')

%% Test Fan-Temperature controller on real system:
% measure and shift with standby temperature
    t_step = 10; % nostep
    t_sim  = 2;
    stepBulb = 0;
    stepFan = 0;
    stepLed = 0;
    open('runSystem');
    set_param(gcs,'StopTime',num2str(t_sim));
    set_param(gcs,'SimulationCommand','start');
    while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
        pause(1)
    end%while
    pause(1) %Waiting for simulink to save to WS
sb_T = Temperature(1,1); %standby temperature
sb_T = 27;
close_system('runSystem');

y0 = sb_T + 10; % Need a higher temperature to test Fan on
t_sim = 800;

fixed_u1 = 0.25;% Needed to get higher T to test Fan on
fixed_u2 = 0;
fixed_u3 = 0;

% Choose IO:
turnOn_u1 = 0; 
turnOn_u2 = 1;  % Fan  
turnOn_u3 = 0;
turnOn_y1 = 1;  % Temperature
turnOn_y2 = 0;

open('runSISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotSISOcontrol(y,u,ys,IAE,time);
% figIm = getframe(gcf);
% imwrite (figIm.cdata, 'SISOtestFT.png', 'png');
% hgexport(fig,'SISOtestFT.eps')

%% Using Bulb as Light Intensity controller. 
% Light Insensity is controlled by a SISO controller on the Bulb and the
% Temperature is not taken into any consideration.

% System transfer function:
sys_tf = tf_BLI;
sys_num = cell2mat(sys_tf.num);        
sys_den = cell2mat(sys_tf.den);    

% Because of unstable light control the controller time constant needs to
% be at least 0.5 sec.
[Kc,tauI] = SIMCPI(sys_tf,0.5);       

% Test controller on estimated system
y0 = 0;             % Shifting of y
y_s0 = 15;          % initial set-point
t_sim = 12;         % Simulation time
stepTime1 = 2;      % Time when first step
stepTime2 = 4;      % Time when second step
step_ys1 = 15;      % step in setpoint
step_ys2 = -15;     % Step in setpoint
stepTime_d1 = 6;    % Time disturbance 1
stepTime_d2 = 8;    % Time disturbance 2
step_d1 = -5;       % Value of disturbance 1
step_d2 = +5;       % Value of disturbance 2

% Test controller on estimated system
open('simSISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotSISOcontrol(y,u,ys,IAE,time);
figIm = getframe(gcf);
imwrite (figIm.cdata, 'SISOsimBLI.png', 'png');
hgexport(fig,'SISOsimBLI.eps')

%% Test Bulb - Light Intensity controller on real system:

% Choose IO
turnOn_u1 = 1; % Bulb in
turnOn_u2 = 0;
turnOn_u3 = 0;
turnOn_y1 = 0; 
turnOn_y2 = 1; % Light Intensity
fixed_u1 = 0;
fixed_u2 = 0;
fixed_u3 = 0;

open('runSISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotSISOcontrol(y,u,ys,IAE,time);
% figIm = getframe(gcf);
% imwrite (figIm.cdata, 'SISOtestBLI.png', 'png');
% hgexport(fig,'SISOtestBLI.eps')

%% Using LED as Light Intensity controller. 
% Light Insensity is controlled by a SISO controller on the ED and the
% Temperature is not taken into any consideration.

% System transfer function:
sys_tf = tf_LLI;
sys_num = cell2mat(sys_tf.num);        
sys_den = cell2mat(sys_tf.den);    
% Because of unstable light control the controller time constant needs to
% be at least 0.5 sec.
[Kc,tauI] = SIMCPI(sys_tf,0.5);       

% Test controller on estimated system
y0 = 0;             % Shifting of y
y_s0 = 4;           % initial set-point
t_sim = 12;         % Simulation time
stepTime1 = 2;      % Time when first step
stepTime2 = 4;      % Time when second step
step_ys1 = 4;       % step in setpoint
step_ys2 = -4;      % Step in setpoint
stepTime_d1 = 6;    % Time disturbance 1
stepTime_d2 = 8;    % Time disturbance 2
step_d1 = -2;       % Value of disturbance 1
step_d2 = +2;       % Value of disturbance 2

% Test controller on estimated system
open('simSISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotSISOcontrol(y,u,ys,IAE,time);
figIm = getframe(gcf);
imwrite (figIm.cdata, 'SISOsimLLI.png', 'png');
hgexport(fig,'SISOsimLLI.eps')

%% Test LED - Light Intensity controller on real system:

% Choose IO
turnOn_u1 = 0; % Bulb in
turnOn_u2 = 0;
turnOn_u3 = 1;
turnOn_y1 = 0; 
turnOn_y2 = 1; % Light Intensity
fixed_u1  = 0;
fixed_u2  = 0;
fixed_u3  = 0;

open('runSISOcontrol');
set_param(gcs,'StopTime',num2str(t_sim));
set_param(gcs,'SimulationCommand','start');
while get_param(gcs,'SimulationStatus')=='running' %Waiting on simulink
    pause(1)
end%while
pause(1) %Waiting for simulink to save to WS

% plot result:
fig = plotSISOcontrol(y,u,ys,IAE,time);
% figIm = getframe(gcf);
% imwrite (figIm.cdata, 'SISOtestLLI.png', 'png');
% hgexport(fig,'SISOtestLLI.eps')
