% Identifies transfer function models, state space models, Gains and RGA
% The data (WS_Bulb.mat, WS_Fan.mat and WS_Led.mat) of the responses 
% was collected in runSteptest.m. 
% The script uses the System Identification Toolbox to estimate the 
% transfer functions. All non-zero responses was estimated as first order
% responses. There was no significant response in Temperature from LED and 
% no response in Light Intensity from the Fan. All delays was neglected.
% 
% See also RUNSTEPTEST.m TFEST SS

% Author: Sindre Johan Heggheim, sindrjh@stud.ntnu.no, November 20th, 2017
% Specialization project TKP4580 at Department of Chemical Engineering

close all
clear 
clc

% figure settings
figProps.OuterPosition = [100 100 1000 500];
figProps.Color = [1 1 1];

%% Loading responses:
load('WS_Bulb')
load('WS_Fan')
load('WS_Led')

Ts = 0.1;
stepIndx_B = 100/Ts;  %100 sec
stepIndx_F = 10/Ts;   %10 sec
stepIndx_L = 10/Ts;  %10 sec
data_BT  = dataBulb(stepIndx_B:end,'Temperature','Bulb');
data_BLI = dataBulb(stepIndx_B:end,'Light','Bulb');
data_FT  = dataFan(stepIndx_F:end,'Temperature','Fan');
data_LLI = dataLed(stepIndx_L:end,'Light','Led');
timeBulb = timeBulb(stepIndx_B:end);
timeFan  = timeFan(stepIndx_F:end);
timeLed  = timeLed(stepIndx_L:end);

%% Bulb to Temperature

% Plot the measured and estimated response:
fig = figure();
set(fig,figProps);
subplot(1,2,1)
plot(timeBulb,data_BT.outputData);
xlabel('Time')
ylabel('Measured Temperature')

% estimated 1st order tf:
tf_BT = tfest(data_BT,1,0);
[T_est,t] = step(tf_BT,timeBulb(end)-timeBulb(1));
subplot(1,2,2)
plot(t,T_est)

% The esimated transfer function has to high gain. The transfer function
% is instead calculated with gain delta_y / delta_u and time constant 
% at 63% of the step
u_init = data_BT.inputData(1);
u_end = data_BT.inputData(end);
y_init = data_BT.outputData(1);
y_end = data_BT.outputData(end);
gain_BT = (y_end-y_init)/(u_end-u_init);
tC_idx=1;
while data_BT.outputData(tC_idx)<y_init+(y_end-y_init)*0.63
    tC_idx=tC_idx+1;
end%while
tConst_BT = timeBulb(tC_idx)-timeBulb(1);
tf_BT_calc = tf([gain_BT],[tConst_BT 1]);

hold on
[T_est,t] = step(tf_BT_calc,timeBulb(end)-timeBulb(1));
plot(t,T_est)
xlabel('Time')
ylabel('Temperature')
legend('Identified tf','Calculated tf','Location','southeast')

% save figure
figIm = getframe(gcf);
imwrite (figIm.cdata, 'BT_tf.png', 'png');
hgexport(fig,'BT_tf.eps')

%% Bulb to Light Intensity
% plot measured and estimated response:
fig = figure();
set(fig,figProps);
subplot(1,2,1)
plot(timeBulb(2:12),data_BLI.outputData(2:12));
xlabel('Time')
ylabel('Measured Light Intensity')

% estimated 1st order tf
subplot(1,2,2) 
tf_BLI = tfest(data_BLI,1,0);
[LI_est,t]=step(tf_BLI,1); % The esimated transfer function is a good fit.
plot(t,LI_est)
xlabel('Time')
ylabel('Light Intensity')

% save figure
figIm = getframe(gcf);
imwrite (figIm.cdata, 'BLI_tf.png', 'png');
hgexport(fig,'BLI_tf.eps')

%% Fan to Temperature
% plot measured and estimated response:
fig = figure();
set(fig,figProps);
subplot(1,2,1)
plot(timeFan,data_FT.outputData);
xlabel('Time')
ylabel('Measured Temperature')

% estimated 1st order tf 
tf_FT = -tfest(data_FT,1,0); % negative response
[T_est,t]=step(tf_FT,[0:Ts:timeFan(end)]);
subplot(1,2,2)
plot(t,T_est)

% The esimated transfer function has to high gain. The transfer function
% is instead calculated with gain delta_y / delta_u and time constant 
% at 63% of the step
hold on
u_init = data_FT.inputData(1);
u_end = data_FT.inputData(end);
y_init = data_FT.outputData(1);
y_end = data_FT.outputData(end);
gain_FT = (y_end-y_init)/(u_end-u_init);

tau1_idx=1;
while data_FT.outputData(tau1_idx)>y_init+(y_end-y_init)*0.63
    tau1_idx=tau1_idx+1;
end%while
tau1_FT = timeFan(tau1_idx)-timeFan(1);
tf_FT_calc = tf([gain_FT],[tau1_FT 1]);
hold on
[T_est,t]=step(tf_FT_calc,[0:Ts:timeFan(end)]);
plot(t,T_est)
legend('tf from ident.','tf calculated','Location','southeast')
xlabel('Time')
ylabel('Temperature')

% save figure
figIm = getframe(gcf);
imwrite (figIm.cdata, 'FT_tf.png', 'png');
hgexport(fig,'FT_tf.eps')

%% Led to Light
% plot measured and estimated response:
fig = figure();
set(fig,figProps);
subplot(1,2,1)
plot(timeLed(2:12),data_LLI.outputData(2:12));
xlabel('Time')
ylabel('Measured Light Intensity')

% estimated 1st order tf
subplot(1,2,2)
tf_LLI = tfest(data_LLI,1,0);
[LI_est,t]=step(tf_LLI,1); % The estimated transfer function is a good fit.
plot(t,LI_est);
ylabel('Light Intensity')
xlabel('Time')
ylim([0,10])

% save figure
figIm = getframe(gcf);
imwrite (figIm.cdata, 'LLI_tf.png', 'png');
hgexport(fig,'LLI_tf.eps')

%% Save transfer functions to .mat file
tf_BT = tf_BT_calc;
tf_BT.InputName = 'Bulb';
tf_BT.OutputName = 'Temperature';
tf_BLI = 1*tf_BLI; % makes idtf to tf
tf_BLI.InputName = 'Bulb';
tf_BLI.OutputName = 'Light';
tf_FT = tf_FT_calc;
tf_FT.InputName = 'Fan';
tf_FT.OutputName = 'Temperature';
tf_LLI = 1*tf_LLI; % makes idtf to tf
tf_LLI.InputName = 'LED';
tf_LLI.OutputName = 'Light';
save('WS_tfs.mat','tf_BT','tf_BLI','tf_FT','tf_LLI')

%% Save MIMO-models to .mat file
MIMO_tf = [tf_BT, tf_FT, 0; tf_BLI, 0, tf_LLI ];
MIMO_css = ss(MIMO_tf);
save('WS_MIMO_models.mat','MIMO_tf','MIMO_css');

%% first order response parameters:
close all
clear
clc
load('WS_tfs')

numden = [cell2mat(tf_BT.num);cell2mat(tf_BT.den)]; %Bulb Temperature
BT_gain = numden(1,2)/numden(2,2);
BT_tau1 = numden(2,1)/numden(2,2);

numden = [cell2mat(tf_FT.num);cell2mat(tf_FT.den)]; % Fan Temperature
FT_gain = numden(1,2)/numden(2,2);
FT_tau1 = numden(2,1)/numden(2,2);

numden = [cell2mat(tf_BLI.num);cell2mat(tf_BLI.den)];% Bulb Light Intensity
BLI_gain = numden(1,2)/numden(2,2);
BLI_tau1 = numden(2,1)/numden(2,2);

numden = [cell2mat(tf_LLI.num);cell2mat(tf_LLI.den)]; % LED Light Intensity
LLI_gain = numden(1,2)/numden(2,2);
LLI_tau1 = numden(2,1)/numden(2,2);

tfParam	 = [  BT_gain,  BT_tau1;...
                FT_gain,  FT_tau1;...
                BLI_gain, BLI_tau1;...
                LLI_gain, LLI_tau1];
            
%% Steady state gains and RGA:
close all
clear all
clc
load('WS_tfs')

G = zeros(2,3);
s=0;

num = cell2mat(tf_BT.num);   
den = cell2mat(tf_BT.den);
G(1,1) = (num(1)*s+num(2))/(den(1)*s+den(2)); % tf_BT

num = cell2mat(tf_FT.num);    
den = cell2mat(tf_FT.den);     
G(1,2) = (num(1)*s+num(2))/(den(1)*s+den(2)); % tf_FT

G(1,3) = 0;

num = cell2mat(tf_BLI.num);    
den = cell2mat(tf_BLI.den);     
G(2,1) = (num(1)*s+num(2))/(den(1)*s+den(2)); % tf_BLI

G(2,2) = 0;
num = cell2mat(tf_LLI.num);    
den = cell2mat(tf_LLI.den);     
G(2,3) = (num(1)*s+num(2))/(den(1)*s+den(2)); % tf_LLI
    
RGA_Matrix =G.*pinv(G.');
save('WS_RGA.mat','RGA_Matrix','G')
